use std::{collections::HashMap, process::exit};

use dialoguer::Select;

use crate::{services::{instagram::Instagram, service::Service}, types::{participant::Participant, project::Project, user::User}};

pub async fn add(path: String) {
    let mut raw_project = Project::load(path).await.unwrap_or_else(|e| {
        eprintln!("Error while loading a project: {}", e);
        exit(1);
    });

    if raw_project.project.users.is_empty() {
        eprintln!("Error! You do not have any users in your project. Create at least one to import chat.");
        exit(1);
    }

    let services: Vec<Box<dyn Service>> = vec![
        Box::new(Instagram { 
            data: None
        }), 
    ];

    let options: Vec<&'static str> = services.iter().map(|service| service.get_name()).collect();

    let selected = Select::new()
        .with_prompt("Choose a service")
        .items(&options)
        .default(0)
        .interact()
        .unwrap();

    let service = services.get(selected).unwrap();
    let mut final_service = service.setup();
    let _ = final_service.load().await;

    let participants = final_service.get_participants();

    let mut user_map: HashMap<Participant, User> = HashMap::new();

    println!("Now select what project user is coresponding to a {} participant", service.get_name());
    for user in raw_project.project.users.clone() {
        let parts: Vec<Participant> = participants.iter().filter(|participant| {
            !user_map.contains_key(&participant)
        }).map(|e| e.clone()).collect();

        let options: Vec<String> = parts.iter().map(|part| part.name.clone()).collect();

        let selected = Select::new()
            .with_prompt(format!("Select a {} participant for user '{}'", service.get_name(), user.name))
            .items(&options)
            .default(0)
            .interact()
            .unwrap();

        let selected_part = parts.get(selected).unwrap().clone();

        user_map.insert(selected_part.clone(), user);
    }

    println!("Starting to merge messages...");
    final_service.merge_messages(&mut raw_project, &user_map)
}
