use std::process::exit;

use dialoguer::{Input, Select};
use tabled::Table;

use crate::types::{project::Project, user::User};

pub async fn users(path: String) {
    let project = Project::load(path.clone()).await.unwrap_or_else(|e| {
        eprintln!("Error while loading a project: {}", e);
        exit(1);
    }).project;

    menu(project, path).await;
}

pub fn print_users(users: &Vec<User>) {
    if users.is_empty() {
        println!("Currently you do not have any users!");
        return
    }

    let users = Table::new(users).to_string();
    println!("{}", users);
}

pub async fn menu(mut project: Project, path: String) {
    println!("User management:");
    print_users(&project.users);
    println!("\n");

    let options = vec!["Create a new user", "Remove a user", "Save and exit"];

    let option = Select::new()
        .with_prompt("Choose a option")
        .items(&options)
        .default(0)
        .interact()
        .unwrap();

    match option {
        0 => {
            // Create a user
            let name: String = Input::new()
                .with_prompt("Name")
                .interact_text()
                .expect("Failed to read a line");

            let user = User {
                name
            };

            project.users.push(user);

            Box::pin(menu(project, path)).await;
        },
        1 => {
            if project.users.is_empty() {
                println!("You do not have any users created! No users to be removed!");
                Box::pin(menu(project, path)).await;
                return;
            }

            let options_users: Vec<String> = project.users.iter().map(|user| {
                user.name.clone()
            }).collect();

            let option_remove = Select::new()
                .with_prompt("Select user to be removed")
                .items(&options_users)
                .default(0)
                .interact()
                .unwrap();
            
            project.users.remove(option_remove);

            Box::pin(menu(project, path)).await;
        },
        2 => {
            // Save and exit
            let _ = project.save(path).await;
        },
        _ => {
            println!("Error! Unknown option!");
            Box::pin(menu(project, path)).await;
        }
    }
}
